#!/bin/python3
import os

import argparse

from .cmds.config import set_config_parser, config_cmd
from .cmds.remove import remove_cmd, set_remove_parser
from .cmds.clear import clear_cmd
from .cmds.infos import set_infos_parser, infos_cmd
from .cmds.use import use_cmd, set_use_parser
from .cmds.download import set_download_parser, download_cmd
from .cmds.list import list_cmd, set_list_parser
from .utils import open_config, create_config

parser = argparse.ArgumentParser(description='Jetbrains toolbox cli', prog="fttb")

subparsers = parser.add_subparsers(dest="command")
list_parser = subparsers.add_parser('list', help='list ides')
set_list_parser(list_parser)
download_parser = subparsers.add_parser('download', help='download an ide')
set_download_parser(download_parser)
use_parser = subparsers.add_parser('use', help='use an ide')
set_use_parser(use_parser)
infos_parser = subparsers.add_parser('infos', help='get infos about an ide')
set_infos_parser(infos_parser)
remove_parser = subparsers.add_parser('remove', help='remove an ide')
set_remove_parser(remove_parser)
config_parser = subparsers.add_parser('config', help='config fttb')
set_config_parser(config_parser)
subparsers.add_parser('clear', help='clear download cache')


def main(cmd_args):
    create_config()
    config_fttb = open_config(f"{os.getenv('HOME')}/.config/fttb/config.json")
    if cmd_args.command == "list":
        list_cmd(cmd_args, config_fttb)
    elif cmd_args.command == "download":
        download_cmd(cmd_args, config_fttb)
    elif cmd_args.command == "use":
        use_cmd(cmd_args, config_fttb)
    elif cmd_args.command == "infos":
        infos_cmd(cmd_args, config_fttb)
    elif cmd_args.command == "clear":
        clear_cmd(config_fttb)
    elif cmd_args.command == "remove":
        remove_cmd(cmd_args, config_fttb)
    elif cmd_args.command == "config":
        config_cmd(cmd_args)


if __name__ == '__main__':
    args = parser.parse_args()
    if os.environ.get("HOME", None) is None:
        print("HOME is unset")
        exit()
    main(args)



import json
import os
from argparse import ArgumentParser
from ..utils import download_file, open_config


def set_config_parser(parser: ArgumentParser):
    parser.add_argument("--cache-path", dest="cache_path", help="set download cache path")
    parser.add_argument("--install-path", dest="install_path", help="set ides installation path")
    parser.add_argument("--bin-path", dest="bin_path", help="set ides binaries path")


def config_cmd(args):
    config_fttb = open_config(f"{os.getenv('HOME')}/.config/fttb/config.json")
    if args.cache_path is not None:
        config_fttb['cache_path'] = args.cache_path
    if args.install_path is not None:
        config_fttb['install_path'] = args.install_path
    if args.bin_path is not None:
        config_fttb['bin_path'] = args.bin_path



import os
import shutil
import sys
import tarfile
from argparse import ArgumentParser

import requests

from ..utils import get_code, parse_version, download_file


def set_download_parser(parser: ArgumentParser):
    parser.add_argument("ide", nargs="?", default="all")
    parser.add_argument("version", nargs='?', default="latest")
    parser.add_argument("--type", choices=["release", "eap", "rc"], default="release")


def download_ide(ide, version, type, config_fttb):
    ide_code = get_code(ide, config_fttb)
    res = requests.get(
        f"https://data.services.jetbrains.com/products?code={ide_code}&fields=releases"
    )
    if not res.ok:
        sys.exit()

    releases = res.json()[0]['releases']
    version = parse_version(ide_code, version, type)

    if os.path.isdir(f"{config_fttb['install_path']}/{ide_code}-{version}"):
        return version
    for release in releases:
        if release['version'] == version and release['type'] == type:
            filename = release['downloads']['linux']['link'].split("/")[-1]
            filepath = f"{config_fttb['cache_path']}/{filename}"
            download_file(release['downloads']['linux']['link'], filepath)
            file = tarfile.open(filepath)
            file.extractall(path=f"{config_fttb['install_path']}/")
            dst = file.getmembers()[0].name.split('/')[0]
            try:
                shutil.rmtree(f"{config_fttb['install_path']}/{ide_code}-{version}")
            except FileNotFoundError:
                pass
            os.rename(f"{config_fttb['install_path']}/{dst}", f"{config_fttb['install_path']}/{ide_code}-{version}")
            return version
    raise Exception


def download_cmd(args, config_fttb):
    if args.ide == "all":
        print("invalid IDE code")
        return
    download_ide(args.ide, args.version, args.type, config_fttb)



import os.path
import pydoc
import sys
from argparse import ArgumentParser

import requests

from ..utils import get_code


def set_list_parser(parser: ArgumentParser):
    parser.add_argument("ide", nargs="?", default="all")
    parser.add_argument("--installed", action='store_true')


def list_cmd(args, config_fttb):
    if args.ide == "all":
        codes = ",".join(config_fttb['aliases'].values())
        res = requests.get(
            f"https://data.services.jetbrains.com/products?fields=name,intellijProductCode,description,categories&code={codes}")

        if not res.ok:
            print("request failed")
            sys.exit()

        ides = res.json()

        for ide in ides:
            if ide['intellijProductCode'] is not None and ide['categories'] is not None:
                if "IDE" in ide['categories']:
                    print(f"{ide['name']}\n{ide['description']}\n")
    else:
        ide_code = get_code(args.ide, config_fttb)
        res = requests.get(
            f"https://data.services.jetbrains.com/products?code={ide_code}&fields=releases")
        if not res.ok:
            print("request failed")
            sys.exit()

        releases = res.json()[0]['releases']
        page = ""
        for release in releases:
            if args.installed:
                if os.path.exists(f"{config_fttb['install_path']}/{ide_code}-{release['version']}"):
                    page += f"{release['date']}\n{release['type']}\n{release['version']}\n\n"
            else:
                page += f"{release['date']}\n{release['type']}\n{release['version']}\n\n"
        pydoc.pager(page)

